"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRedisClientListProvider = exports.createRedisClientProvider = void 0;
const ioredis_1 = __importDefault(require("ioredis"));
const common_constant_1 = require("../constants/common.constant");
const redis_constant_1 = require("../constants/redis.constant");
const redis_service_1 = require("../services/redis.service");
const redis_util_1 = require("../utils/redis.util");
/**
 * 创建Redis客户端列表Provider，多个Provider，Inject Token 为 namespace
 * @returns Redis客户端列表Provider
 */
const createRedisClientProvider = () => common_constant_1.namespaces.map(namespace => ({
    provide: namespace,
    useFactory: (redisUtil) => redisUtil.getClient(namespace),
    inject: [redis_util_1.RedisUtil]
}));
exports.createRedisClientProvider = createRedisClientProvider;
/**
 * 创建Redis客户端列表 FactoryProvider，单个Provider，Inject Token 为 REDIS_CLIENT_LIST_PROVIDER
 * @returns Redis客户端列表 FactoryProvider
 */
const createRedisClientListProvider = () => ({
    provide: redis_constant_1.REDIS_CLIENT_LIST_PROVIDER,
    useFactory: ({ defaultOptions, config }) => {
        if (!config) {
            // namespaces.push(DEFAULT_REDIS_CLIENT_PROVIDER);
            common_constant_1.redisClientListMap.set(redis_constant_1.DEFAULT_REDIS_CLIENT_PROVIDER, createClient(defaultOptions));
        }
        if (Array.isArray(config)) {
            for (const option of config) {
                common_constant_1.redisClientListMap.set(option.namespace ?? redis_constant_1.DEFAULT_REDIS_CLIENT_PROVIDER, createClient({ ...defaultOptions, ...option }));
                // namespaces.push(option.namespace ?? DEFAULT_REDIS_CLIENT_PROVIDER);
            }
        }
        else {
            common_constant_1.redisClientListMap.set(config.namespace ?? redis_constant_1.DEFAULT_REDIS_CLIENT_PROVIDER, createClient({ ...defaultOptions, ...config }));
            // namespaces.push(config.namespace ?? DEFAULT_REDIS_CLIENT_PROVIDER);
        }
        return common_constant_1.redisClientListMap;
    },
    inject: [common_constant_1.OPTIONS_PROVIDER]
});
exports.createRedisClientListProvider = createRedisClientListProvider;
/**
 * 创建Redis客户端
 * @param clientOptions 客户端配置信息
 * @returns Redis 客户端
 */
const createClient = (clientOptions) => {
    const { onClientCreated, ...redisOptions } = clientOptions;
    const client = new ioredis_1.default(redisOptions);
    common_constant_1.serviceListWeakMap.set(client, new redis_service_1.RedisService(client, redisOptions));
    if (onClientCreated)
        onClientCreated(client);
    return client;
};
